/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gtk/gtk.h>

#include <sensors/sensors.h>
#include <sensors/error.h>

#include "psensor.h"
#include "graph.h"
#include "ui_sensorlist.h"
#include "nvidia.h"

#define PSENSOR_VERSION "0.2.1"

struct ui_psensor {	
	struct psensor** sensors;

	GtkWidget* w_graph;

	struct ui_sensorlist* ui_sensorlist;	
};

void update_information(struct psensor** sensors) 
{
	while(1) {
		struct psensor** sensor_cur = sensors;
		while(*sensor_cur) {
			gdk_threads_enter();
			update_psensor(*sensor_cur);
			gdk_threads_leave();
			sensor_cur++;
		}
		sleep(1);
	}
}

gboolean graph_refresh_thread(gpointer data) 
{
	struct ui_psensor* ui_psensor = (struct ui_psensor*)data;

	update_graph(ui_psensor->sensors,
		     ui_psensor->w_graph);

	ui_sensorlist_update(ui_psensor->ui_sensorlist);

	return TRUE;
}



gboolean on_expose_event (GtkWidget *widget, 
			  GdkEventExpose *event, 
			  gpointer data) 
{
	struct ui_psensor* ui_psensor = (struct ui_psensor*)data;

	update_graph(ui_psensor->sensors, 
		     ui_psensor->w_graph);

	return FALSE;
}

void on_destroy(GtkWidget *widget,
	     gpointer data )
{
    gtk_main_quit ();
}


int main(int argc, char** argv) 
{
	struct ui_psensor ui_psensor;

	printf("Psensor - Temperature Monitor - v%s by wpitchoune@gmail.com\n",
	       PSENSOR_VERSION);
	printf("Released under GPLv2 license terms, read LICENSE file\n");
	printf("http://wpitchoune.net:9969/psensor\n");

	int err = sensors_init(NULL);
	if (err) {
		fprintf(stderr, "ERROR: %s\n", sensors_strerror(err));
		return 1;
	}

	ui_psensor.sensors = get_all_sensors();


	g_thread_init(NULL);
	gdk_threads_init();
	gdk_threads_enter();

	gtk_init (&argc, &argv);

	GtkWidget* window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_container_set_border_width (GTK_CONTAINER (window), 2);
	gtk_window_set_default_size(GTK_WINDOW(window),800,200);
	gtk_window_set_title(GTK_WINDOW(window), 
			     "Psensor - Temperature Monitor");
	
	g_signal_connect (window, "destroy",
			  G_CALLBACK(on_destroy),
			  NULL);

	//////////// main box in the window
	GtkWidget* mbox = gtk_hbox_new(FALSE,0);
	gtk_container_add (GTK_CONTAINER (window), mbox);

	GtkDrawingArea* drawing_area = GTK_DRAWING_AREA(gtk_drawing_area_new ());
	ui_psensor.w_graph = GTK_WIDGET(drawing_area);
	gtk_box_pack_start(GTK_BOX(mbox), GTK_WIDGET(drawing_area), TRUE, TRUE, 0);

	g_signal_connect(G_OBJECT(drawing_area), 
			 "expose-event",
			 G_CALLBACK(on_expose_event), &ui_psensor);

	///////////// sensor list
	ui_psensor.ui_sensorlist = ui_sensorlist_create(ui_psensor.sensors);
	gtk_box_pack_start(GTK_BOX(mbox), 
			   GTK_WIDGET(ui_psensor.ui_sensorlist->widget), 
			   FALSE, FALSE, 0);     

	gtk_widget_show_all (window);

	GError* error;
	GThread* thread = g_thread_create((GThreadFunc)update_information,
					  ui_psensor.sensors, TRUE, &error);

	if(!thread) 
		g_error_free(error);
	
	g_timeout_add (1000, graph_refresh_thread, &ui_psensor);

	gtk_main ();
	gdk_threads_leave();

	sensors_cleanup();

	psensor_list_free(ui_psensor.sensors);

	return 0;
}
